@@ -1,6 +1,6 @@
 /* GMP-ECM -- Integer factorization with ECM and Pollard 'P-1' methods.
 
-  Copyright 2001, 2002, 2003, 2004, 2005 Jim Fougeron, Laurent Fousse, Alexander Kruppa, Paul Zimmermann.
+  Copyright 2001, 2002, 2003, 2004, 2005, 2011 Jim Fougeron, Laurent Fousse, Alexander Kruppa, Paul Zimmermann.
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -26,6 +26,7 @@
 #ifdef _MSC_VER
 #  include <winsock2.h>
 #endif
+#include "ecm-impl.h"
 #include "ecm-ecm.h"
 
 #ifdef HAVE_UNISTD_H /* for access() */
@@ -46,19 +47,12 @@
 #include "gwnum.h"
 #endif
 
+/* Used in print_config() */
+#include "ecm-params.h"
+
 /* #define DEBUG */
 
-/* people keeping track of champions and corresponding url's: ECM, P-1, P+1 */
-static char *champion_keeper[3] =
-{ "Richard Brent <champs@rpbrent.com>",
-  "Paul Zimmermann <zimmerma@loria.fr>",
-  "Paul Zimmermann <zimmerma@loria.fr>"};
-static char *champion_url[3] =
-{"http://wwwmaths.anu.edu.au/~brent/ftp/champs.txt",
- "http://www.loria.fr/~zimmerma/records/Pminus1.html",
- "http://www.loria.fr/~zimmerma/records/Pplus1.html"};
-/* minimal number of digits to enter the champions table for ECM, P-1, P+1 */
-static unsigned int champion_digits[3] = { 61, 50, 40 };
+#include "champions.h"
 
 /* probab_prime_p() can get called from other modules. Instead of passing
    prpcmd to those functions, we make it static here - this variable will
@@ -73,20 +67,20 @@ static int exit_asap_signalnr = 0; /* Remembers which signal we received */
 /* Tries to read a number from a line from fd and stores it in r.
    Keeps reading lines until a number is found. Lines beginning with "#"
      are skipped.
-   Returns 1 if a number was successfully read, 0 if no number can be read 
+   Returns 1 if a number was successfully read, 0 if no number can be read
      (i.e. at EOF)
    Function is now simpler.  Much of the logic (other than skipping # lines
      is now contained within eval() function.
 */
 
-int 
+int
 read_number (mpcandi_t *n, FILE *fd, int primetest)
 {
   int c;
-  
+
 new_line:
   c = fgetc (fd);
-  
+
   /* Skip comment lines beginning with '#' */
   if (c == '#')
     {
@@ -129,7 +123,7 @@ new_line:
   return 1;
 }
 
-int 
+int
 probab_prime_p (mpz_t N, int reps)
 {
 #ifdef WANT_SHELLCMD
@@ -155,15 +149,15 @@ probab_prime_p (mpz_t N, int reps)
       return mpz_probab_prime_p (N, reps);
 }
 
-void 
+void
 signal_handler (int sig)
 {
   if (sig == SIGINT || sig == SIGTERM)
     {
       exit_asap_value = 1;
       exit_asap_signalnr = sig;
-      /* If one of these two signals arrives again, we'll let the default 
-         handler take over,  which will usually terminate the process 
+      /* If one of these two signals arrives again, we'll let the default
+         handler take over,  which will usually terminate the process
          immediately. */
       signal (SIGINT, SIG_DFL);
       signal (SIGTERM, SIG_DFL);
@@ -180,8 +174,7 @@ stop_asap_test ()
   return exit_asap_value;
 }
 
-
-static void 
+static void
 usage (void)
 {
     printf ("Usage: ecm [options] B1 [[B2min-]B2] < file\n");
@@ -205,6 +198,7 @@ usage (void)
     printf ("  -modmuln     use Montgomery's MODMULN for modular reduction\n");
     printf ("  -redc        use Montgomery's REDC for modular reduction\n");
     printf ("  -nobase2     disable special base-2 code\n");
+    printf ("  -nobase2s2   disable special base-2 code in ecm stage 2 only\n");
     printf ("  -base2 n     force base 2 mode with 2^n+1 (n>0) or 2^|n|-1 (n<0)\n");
     printf ("  -ntt         enable NTT convolution routines in stage 2\n");
     printf ("  -no-ntt      disable NTT convolution routines in stage 2\n");
@@ -238,11 +232,165 @@ usage (void)
     printf ("  -B2scale f   Multiplies the default B2 value by f \n");
     printf ("  -go val      Preload with group order val, which can be a simple expression,\n");
     printf ("               or can use N as a placeholder for the number being factored.\n");
+    printf ("  -printconfig Print compile-time configuration and exit.\n");
 
+    printf ("  -batch       (experimental) use Montgomery parametrization and batch\n" 
+					  "               computation.\n");
 
     printf ("  -h, --help   Prints this help and exit.\n");
 }
 
+/* Print parameters that were used to build GMP-ECM */
+static void
+print_config ()
+{
+  printf ("Compilation options:\n");
+#ifdef __MPIR_VERSION
+     printf ("Included MPIR header files version %d.%d.%d\n", 
+             __MPIR_VERSION, __MPIR_VERSION_MINOR, __MPIR_VERSION_PATCHLEVEL);
+#else /* __MPIR_VERSION */
+   #ifdef __GNU_MP_VERSION_PATCHLEVEL
+     printf ("Included GMP header files version %d.%d.%d\n", 
+             __GNU_MP_VERSION, __GNU_MP_VERSION_MINOR, 
+             __GNU_MP_VERSION_PATCHLEVEL);
+   #else
+     printf ("Included GMP header files version %d.%d\n", 
+             __GNU_MP_VERSION, __GNU_MP_VERSION_MINOR);
+   #endif
+#endif /* __MPIR_VERSION */
+
+#ifdef GWNUM_VERSION
+  printf ("Included GWNUM header files version %s\n", GWNUM_VERSION);
+#else
+  printf ("GWNUM_VERSION undefined\n");
+#endif
+
+#ifdef HAVE_SSE2
+  printf ("HAVE_SSE2 = %d\n$", HAVE_SSE2);
+#else
+  printf ("HAVE_SSE2 undefined\n");
+#endif
+
+#ifdef HAVE___GMPN_ADD_NC
+  printf ("HAVE___GMPN_ADD_NC = %d\n", HAVE___GMPN_ADD_NC);
+#else
+  printf ("HAVE___GMPN_ADD_NC undefined\n");
+#endif
+
+#ifdef HAVE___GMPN_MOD_34LSUB1
+  printf ("HAVE___GMPN_MOD_34LSUB1 = %d\n", HAVE___GMPN_MOD_34LSUB1);
+#else
+  printf ("HAVE___GMPN_MOD_34LSUB1 undefined\n");
+#endif
+
+#ifdef HAVE___GMPN_REDC_1
+  printf ("HAVE___GMPN_REDC_1 = %d\n", HAVE___GMPN_REDC_1);
+#else
+  printf ("HAVE___GMPN_REDC_1 undefined\n");
+#endif
+
+#ifdef MEMORY_DEBUG
+  printf ("MEMORY_DEBUG = %d\n", MEMORY_DEBUG);
+#else
+  printf ("MEMORY_DEBUG undefined\n");
+#endif
+
+#ifdef USE_ASM_REDC
+  printf ("USE_ASM_REDC = %d\n", USE_ASM_REDC);
+#ifdef TUNE_MULREDC_THRESH
+  printf ("TUNE_MULREDC_THRESH = %d\n", TUNE_MULREDC_THRESH);
+#else
+  printf ("TUNE_MULREDC_THRESH undefined\n");
+#endif
+#ifdef TUNE_SQRREDC_THRESH
+  printf ("TUNE_SQRREDC_THRESH = %d\n", TUNE_SQRREDC_THRESH);
+#else
+  printf ("TUNE_SQRREDC_THRESH undefined\n");
+#endif
+#ifdef WINDOWS64_ABI
+  printf ("WINDOWS64_ABI = %d\n", WINDOWS64_ABI);
+#else
+  printf ("WINDOWS64_ABI undefined\n");
+#endif
+#else
+  printf ("USE_ASM_REDC undefined\n");
+#endif
+
+#ifdef WANT_ASSERT
+  printf ("WANT_ASSERT = %d\n", WANT_ASSERT);
+#else
+  printf ("WANT_ASSERT undefined\n");
+#endif
+
+#ifdef WANT_SHELLCMD
+  printf ("WANT_SHELLCMD = %d\n", WANT_SHELLCMD);
+#else
+  printf ("WANT_SHELLCMD undefined\n");
+#endif
+
+#ifdef _OPENMP
+  printf ("_OPENMP = %d\n", _OPENMP);
+#else
+  printf ("_OPENMP undefined\n");
+#endif
+
+#ifdef MPZMOD_THRESHOLD
+  printf ("MPZMOD_THRESHOLD = %d\n", MPZMOD_THRESHOLD);
+#else
+  printf ("MPZMOD_THRESHOLD undefined\n");
+#endif
+
+#ifdef REDC_THRESHOLD
+  printf ("REDC_THRESHOLD = %d\n", REDC_THRESHOLD);
+#else
+  printf ("REDC_THRESHOLD undefined\n");
+#endif
+
+#ifdef MUL_NTT_THRESHOLD
+  printf ("MUL_NTT_THRESHOLD = %d\n", MUL_NTT_THRESHOLD);
+#else
+  printf ("MUL_NTT_THRESHOLD undefined\n");
+#endif
+
+#ifdef NTT_GFP_TWIDDLE_DIF_BREAKOVER
+  printf ("NTT_GFP_TWIDDLE_DIF_BREAKOVER = %d\n", 
+	  NTT_GFP_TWIDDLE_DIF_BREAKOVER);
+#else
+  printf ("NTT_GFP_TWIDDLE_DIF_BREAKOVER undefined\n");
+#endif
+
+#ifdef NTT_GFP_TWIDDLE_DIT_BREAKOVER
+  printf ("NTT_GFP_TWIDDLE_DIT_BREAKOVER = %d\n", 
+	  NTT_GFP_TWIDDLE_DIT_BREAKOVER);
+#else
+  printf ("NTT_GFP_TWIDDLE_DIT_BREAKOVER undefined\n");
+#endif
+
+#ifdef PREREVERTDIVISION_NTT_THRESHOLD
+  printf ("PREREVERTDIVISION_NTT_THRESHOLD = %d\n", 
+          PREREVERTDIVISION_NTT_THRESHOLD);
+#else
+  printf ("PREREVERTDIVISION_NTT_THRESHOLD undefined\n");
+#endif
+
+#ifdef POLYINVERT_NTT_THRESHOLD
+  printf ("POLYINVERT_NTT_THRESHOLD = %d\n", POLYINVERT_NTT_THRESHOLD);
+#else
+  printf ("POLYINVERT_NTT_THRESHOLD undefined\n");
+#endif
+
+#ifdef POLYEVALT_NTT_THRESHOLD
+  printf ("POLYEVALT_NTT_THRESHOLD = %d\n", POLYEVALT_NTT_THRESHOLD);
+#else
+  printf ("POLYEVALT_NTT_THRESHOLD undefined\n");
+#endif
+
+#ifdef MPZSPV_NORMALISE_STRIDE
+  printf ("MPZSPV_NORMALISE_STRIDE = %d\n", MPZSPV_NORMALISE_STRIDE);
+#else
+  printf ("MPZSPV_NORMALISE_STRIDE undefined\n");
+#endif
+}
 
 
 /******************************************************************************
@@ -272,6 +420,7 @@ main (int argc, char *argv[])
         /* If a factor was found, indicate whether factor, cofactor are */
         /* prime. If no factor was found, both are zero. */
   int repr = ECM_MOD_DEFAULT; /* automatic choice */
+  int nobase2step2 = 0; /* flag to turn off base 2 arithmetic in ecm stage 2 */
   unsigned long k = ECM_DEFAULT_K; /* default number of blocks in stage 2 */
   int S = ECM_DEFAULT_S;
              /* Degree for Brent-Suyama extension requested by user.
@@ -282,7 +431,7 @@ main (int argc, char *argv[])
   char *savefilename = NULL, *resumefilename = NULL, *infilename = NULL;
   char *TreeFilename = NULL, *chkfilename = NULL;
   char rtime[256] = "", who[256] = "", comment[256] = "", program[256] = "";
-  FILE *savefile = NULL, *resumefile = NULL, *infile = NULL;
+  FILE *resumefile = NULL, *infile = NULL;
   mpz_t resume_lastN, resume_lastfac; /* When resuming residues from a file,
         store the last number processed and the factors found for this it */
   int resume_wasPrp = 0; /* 1 if resume_lastN/resume_lastfac is a PRP */
@@ -304,10 +453,17 @@ main (int argc, char *argv[])
   double maxmem = 0.;
   double stage1time = 0.;
   ecm_params params;
+  int batch = 0; /* By default we don't use batch mode */
 #ifdef WANT_SHELLCMD
   char *faccmd = NULL;
   char *idlecmd = NULL;
 #endif
+#ifdef HAVE_GWNUM
+  double gw_k = 0.0;       /* set default values for gwnum poly k*b^n+c */
+  unsigned long gw_b = 0;  /* set default values for gwnum poly k*b^n+c */
+  unsigned long gw_n = 0;  /* set default values for gwnum poly k*b^n+c */
+  signed long gw_c = 0;    /* set default values for gwnum poly k*b^n+c */
+#endif
 
   /* check ecm is linked with a compatible library */
   if (mp_bits_per_limb != GMP_NUMB_BITS)
@@ -391,6 +547,12 @@ main (int argc, char *argv[])
 	  argv++;
 	  argc--;
         }
+      else if (strcmp (argv[1], "-nobase2s2") == 0)
+        {
+          nobase2step2 = 1;
+          argv++;
+          argc--;
+        }
       else if (strcmp (argv[1], "-ntt") == 0)
 	{
 	  use_ntt = 2; /* Use NTT, even for large input numbers */
@@ -421,10 +583,21 @@ main (int argc, char *argv[])
 	  argv++;
 	  argc--;
         }
+      else if (strcmp (argv[1], "-batch") == 0)
+        {
+	  batch = 1;
+	  argv++;
+	  argc--;
+        }
       else if (strcmp (argv[1], "-h") == 0 || strcmp (argv[1], "--help") == 0)
         {
           usage ();
-          exit (0);
+          exit (EXIT_SUCCESS);
+        }
+      else if (strcmp (argv[1], "-printconfig") == 0)
+        {
+          print_config ();
+          exit (EXIT_SUCCESS);
         }
       else if (strcmp (argv[1], "-d") == 0)
         {
@@ -635,8 +808,16 @@ main (int argc, char *argv[])
       else if ((argc > 2) && (strcmp (argv[1], "-go") == 0))
 	{
 	  if (go.cpOrigExpr)
-	    free (go.cpOrigExpr);
+            {
+              fprintf (stderr, "Warning, for multiple -go options, only the last one is taken into account.\n");
+              free (go.cpOrigExpr);
+            }
 	  go.cpOrigExpr = malloc (strlen (argv[2]) + 1);
+          if (go.cpOrigExpr == NULL)
+            {
+              fprintf (stderr, "Cannot allocate memory in main\n");
+              exit (1);
+            }
 	  strcpy (go.cpOrigExpr, argv[2]);
 	  if (strchr (go.cpOrigExpr, 'N'))
 	    {
@@ -703,12 +884,32 @@ main (int argc, char *argv[])
   /* start of the program */
   if (verbose >= 1)
     {
+      char Gmp_version[64];
+      char out0[128], *out = out0;
+
+#ifdef __MPIR_VERSION
+      sprintf (Gmp_version, "MPIR %d.%d.%d", __MPIR_VERSION,
+	       __MPIR_VERSION_MINOR, __MPIR_VERSION_PATCHLEVEL);
+#else /* original GMP */
+      sprintf (Gmp_version, "GMP %s", gmp_version);
+#endif /* __MPIR_VERSION */
+
+      out += sprintf (out, "GMP-ECM %s [configured with %s",
+                      VERSION, Gmp_version);
+
 #ifdef HAVE_GWNUM
-      printf ("GMP-ECM %s [powered by GMP %s and GWNUM %s] [", 
-              VERSION, gmp_version, GWNUM_VERSION);
-#else
-      printf ("GMP-ECM %s [powered by GMP %s] [", VERSION, gmp_version);
+      out += sprintf (out, ", GWNUM %s", GWNUM_VERSION);
 #endif
+
+#ifdef USE_ASM_REDC
+      out += sprintf (out, ", --enable-asm-redc");
+#endif
+
+#ifdef WANT_ASSERT
+      out += sprintf (out, ", --enable-assert");
+#endif
+
+      printf ("%s] [", out0);
       switch (method)
 	{
 	case ECM_PM1:
@@ -857,6 +1058,7 @@ main (int argc, char *argv[])
   params->k = k;
   params->S = S;
   params->repr = repr;
+  params->nobase2step2 = nobase2step2;
   params->chkfilename = chkfilename;
   params->TreeFilename = TreeFilename;
   params->maxmem = maxmem;
@@ -884,6 +1086,7 @@ main (int argc, char *argv[])
   /* Open save file for writing, if saving is requested */
   if (savefilename != NULL)
     {
+      FILE *savefile;
       /* Are we not appending and does this file already exist ? */
       if (!saveappend && access (savefilename, F_OK) == 0)
         {
@@ -891,12 +1094,14 @@ main (int argc, char *argv[])
                   savefilename);
           exit (EXIT_FAILURE);
         }
-      savefile = fopen (savefilename, saveappend ? "a" : "w");
+      /* Test if we can open the file for writing */
+      savefile = fopen (savefilename, "a");
       if (savefile == NULL)
         {
           fprintf (stderr, "Could not open file %s for writing\n", savefilename);
           exit (EXIT_FAILURE);
         }
+      fclose (savefile);
     }
 
   if (resumefile && (specific_sigma || mpz_sgn (A) || specific_x0))
@@ -923,7 +1128,7 @@ main (int argc, char *argv[])
      we could save by exiting cleanly, but the waiting for the code to check
      for signals may delay program end unacceptably */
 
-  if (savefile)
+  if (savefilename != NULL)
     {
       signal (SIGINT, &signal_handler);
       signal (SIGTERM, &signal_handler);
@@ -1010,10 +1215,12 @@ BreadthFirstDoAgain:;
 	}
     }
 
-  while (((breadthfirst && linenum < nCandidates) || feof (infile) == 0)
-         && !exit_asap_value)
+  while (((breadthfirst && linenum < nCandidates) || cnt > 0 || 
+          feof (infile) == 0) && !exit_asap_value)
     {
       trial_factor_found = 0;
+      params->B1done = B1done; /* may change with resume */
+      
       if (resumefile != NULL) /* resume case */
         {
 	  if (count != 1)
@@ -1021,9 +1228,8 @@ BreadthFirstDoAgain:;
 	      fprintf (stderr, "Error, option -c and -resume are incompatible\n");
 	      exit (EXIT_FAILURE);
 	    }
-
           if (!read_resumefile_line (&method, x, &n, sigma, A, orig_x0, 
-                &B1done, program, who, rtime, comment, resumefile))
+                &(params->B1done), program, who, rtime, comment, resumefile))
             break;
           
           if (mpz_cmp (n.n, resume_lastN) == 0)
@@ -1254,25 +1460,105 @@ BreadthFirstDoAgain:;
       cnt --; /* one more curve performed */
 
       mpgocandi_fixup_with_N (&go, &n);
+      /* If we are in batch mode:
+         If A was given one should check that d fits in one word and that x0=2.
+         If A was not given one chooses it at random (and if x0 exists
+         it must be 2). */
+      if (batch == 1)
+        {
+          static int random = 0; /* non-zero if user asked for random curves */
 
+          if (method != ECM_ECM)
+            {
+              fprintf (stderr, "Error, the -batch option is only valid for ECM\n");
+              exit (EXIT_FAILURE);
+            }
+          mpz_set_ui (sigma, 0); 
+          if (random || (mpz_sgn (A) == 0)) /* A was not given by the user */
+            {
+              random = 1;
+              /* We need that d = (A+2)/4 is smaller than 2^GMP_NUMB_BITS */
+              mpz_urandomb (A, randstate, 32);  /* generates d */
+              if (GMP_NUMB_BITS >= 64)
+                mpz_mul (A, A, A);              /* ensures d is a square,
+                                                   which increases the success
+                                                   probability */
+              mpz_mul_2exp (A, A, 2);           /* 4d */
+              mpz_sub_ui (A, A, 2);             /* 4d-2 */
+            }
+          
+          if (mpz_sgn (orig_x0) == 0)
+            mpz_set_ui (orig_x0, 2);
+          else if (mpz_cmp_ui (orig_x0, 2) != 0)
+            {
+              fprintf (stderr, "Error, x0 should be equal to 2"
+                       " in batch mode.\n");
+              exit (EXIT_FAILURE);
+            }
+          
+          mpz_set (x, orig_x0);
+        }
       /* set parameters that may change from one curve to another */
+      params->batch = batch;
+      if (params->batch == 1 && params->batch_B1 != B1)
+        {
+          int st;
+          params->batch_B1 = B1;
+
+          if (verbose > OUTPUT_NORMAL)
+            printf ("Batch mode: \n");
+          st = cputime ();
+          /* construct the batch exponent */
+          compute_s (params->batch_s, params->batch_B1);
+          if (verbose > OUTPUT_NORMAL)
+            printf ("  computing prime product of %lu bits took %ldms\n",
+                    mpz_sizeinbase (params->batch_s, 2), cputime () - st);
+        }
       params->method = method; /* may change with resume */
       mpz_set (params->x, x); /* may change with resume */
       /* if sigma is zero, then we use the A value instead */
-      params->sigma_is_A = mpz_sgn (sigma) == 0;
+      params->sigma_is_A = ((mpz_sgn (sigma) == 0 || batch==1)? 1 : 0);
       mpz_set (params->sigma, (params->sigma_is_A) ? A : sigma);
       mpz_set (params->go, go.Candi.n); /* may change if contains N */
-      params->B1done = B1done; /* may change with resume */
       mpz_set (params->B2min, B2min); /* may change with -c */
       /* Here's an ugly hack to pass B2scale to the library somehow.
          It gets piggy-backed onto B1done */
-      params->B1done = B1done + floor (B2scale * 128.) / 134217728.; 
+      params->B1done = params->B1done + floor (B2scale * 128.) / 134217728.; 
       /* Default, for P-1/P+1 with old stage 2 and ECM, use NTT only 
          for small input */
       if (use_ntt == 1 && (method == ECM_ECM || S != ECM_DEFAULT_S)) 
         params->use_ntt = (mpz_size (n.n) <= NTT_SIZE_THRESHOLD);
       else 
-        params->use_ntt = (use_ntt != 0);
+        params->use_ntt = use_ntt;
+
+#ifdef HAVE_GWNUM
+      /* check if the input number can be represented as k*b^n+c */
+      if (kbnc_z (&gw_k, &gw_b, &gw_n, &gw_c, n.n))
+        {
+          params->gw_k = gw_k;
+          params->gw_b = gw_b;
+          params->gw_n = gw_n;
+          params->gw_c = gw_c;
+          if (verbose > OUTPUT_NORMAL)
+            printf ("Found number: %.0f*%lu^%lu + %ld\n",
+                    gw_k, gw_b, gw_n, gw_c);
+        }
+      else if (kbnc_str (&gw_k, &gw_b, &gw_n, &gw_c, n.cpExpr, n.n))
+        {
+          params->gw_k = gw_k;
+          params->gw_b = gw_b;
+          params->gw_n = gw_n;
+          params->gw_c = gw_c;
+          if (verbose > OUTPUT_NORMAL)
+            printf ("Found number: %.0f*%lu^%lu + %ld\n",
+                    gw_k, gw_b, gw_n, gw_c);
+        }
+      else
+        {
+          if (verbose > OUTPUT_NORMAL)
+            printf ("Did not find a gwnum poly for the input number.\n");
+        }
+#endif
 
 #ifdef WANT_SHELLCMD
       /* See if the system is currently idle, if -idlecmd was given */
@@ -1305,6 +1591,18 @@ BreadthFirstDoAgain:;
           printf ("[%.24s]\n", ctime (&t));
         }
 
+#if 0
+      /* Test mpres_muldivbysomething_si() which is not called in normal
+         operation */
+      mpmod_selftest (n.n);
+#endif
+      
+      if (mpz_cmp_ui (n.n, 0) <= 0)
+	{
+	  fprintf (stderr, "Error, input number should be positive\n");
+	  exit (EXIT_FAILURE);
+	}
+
       /* now call the ecm library */
       result = ecm_factor (f, n.n, B1, params);
 
@@ -1332,8 +1630,8 @@ BreadthFirstDoAgain:;
 	  if (verbose > 0)
             printf ("\n");
 
-          /* Complain about non-proper factors (1, 0, negative) */
-          if (mpz_cmp_ui (f, 2) < 0)
+          /* Complain about non-proper factors (0, negative) */
+          if (mpz_cmp_ui (f, 1) < 0)
             {
               fprintf (stderr, "Error: factor found is ");
               mpz_out_str (stderr, 10, f);
@@ -1460,25 +1758,16 @@ OutputFactorStuff:;
 
       /* Write composite cofactors to savefile if requested */
       /* If no factor was found, we consider cofactor composite and write it */
-      if (savefile != NULL && !n.isPrp)
+      if (savefilename != NULL && !n.isPrp)
         {
           mpz_mod (x, params->x, n.n); /* Reduce stage 1 residue wrt new co-
                                           factor, in case a factor was found */
           /* We write the B1done value to the safe file. This requires that
              a correct B1done is returned by the factoring functions */
-          write_resumefile_line (savefile, method, params->B1done, sigma, 
+          write_resumefile_line (savefilename, method, params->B1done, sigma, 
                                  A, x, &n, orig_x0, comment);
         }
 
-      /* Clean up any temp file left over.  At this time, we assume that if 
-         the user wants to resume the run, then they used -save file.  The 
-         temp save was ONLY to help in case of a power outage (or similar) 
-         for a long run.  It would allow finishing the current candidate, 
-         keeping the existing work done.   Now, we assume we are done. */
-#if !defined (DEBUG_AUTO_SAVE)
-      kill_temp_resume_file ();
-#endif
-
       /* advance B1, if autoincrement value had been set during command line parsing */
       if (!breadthfirst && autoincrementB1 > 0.0)
 	{
@@ -1500,8 +1789,6 @@ OutputFactorStuff:;
 
   if (infilename)	/* infile might be stdin, don't fclose that! */
     fclose (infile);
-  if (savefile)
-    fclose (savefile);
   if (resumefile)
     {
       fclose (resumefile);
