@@ -1,6 +1,7 @@
 /* Pollard 'P-1' algorithm.
 
-  Copyright 2001, 2002, 2003, 2004, 2005 Paul Zimmermann and Alexander Kruppa.
+  Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
+  Paul Zimmermann and Alexander Kruppa.
 
   This file is part of the ECM Library.
 
@@ -115,6 +116,11 @@ mulcascade_mul_d (mul_casc *c, const double n, ATTRIBUTE_UNUSED mpz_t t)
   
   i = c->size++;
   c->val = (mpz_t*) realloc (c->val, c->size * sizeof (mpz_t));
+  if (c->val == NULL)
+    {
+      fprintf (stderr, "Cannot allocate memory in mulcascade_mul_d\n");
+      exit (1);
+    }
   mpz_init (c->val[i]);
   mpz_swap (c->val[i], c->val[i-1]);
 
@@ -153,6 +159,11 @@ mulcascade_mul (mul_casc *c, mpz_t n)
   
   i = c->size++;
   c->val = (mpz_t*) realloc (c->val, c->size * sizeof (mpz_t));
+  if (c->val == NULL)
+    {
+      fprintf (stderr, "Cannot allocate memory in mulcascade_mul\n");
+      exit (1);
+    }
   mpz_init (c->val[i]);
   mpz_swap (c->val[i], c->val[i-1]);
 
@@ -748,6 +759,30 @@ pm1_rootsG (mpz_t f, listz_t G, unsigned long dF, pm1_roots_state_t *state,
 }
 
 
+static void
+print_prob (double B1, const mpz_t B2, unsigned long dF, unsigned long k, 
+            int S, const mpz_t go)
+{
+  double prob;
+  int i;
+  char sep;
+
+  outputf (OUTPUT_VERBOSE, "Probability of finding a factor of n digits:\n");
+  if (go != NULL && mpz_cmp_ui (go, 1UL) <= 0)
+    outputf (OUTPUT_VERBOSE, 
+             "(Use -go parameter to specify known factors in P-1)\n");
+  outputf (OUTPUT_VERBOSE, "20\t25\t30\t35\t40\t45\t50\t55\t60\t65\n");
+  for (i = 20; i <= 65; i += 5)
+    {
+      sep = (i < 65) ? '\t' : '\n';
+      prob = pm1prob (B1, mpz_get_d (B2),
+                      pow (10., i - .5), (double) dF * dF * k, S, go);
+      outputf (OUTPUT_VERBOSE, "%.2g%c", prob, sep);
+    }
+}
+
+
+
 /******************************************************************************
 *                                                                             *
 *                                Pollard P-1                                  *
@@ -807,8 +842,14 @@ pm1 (mpz_t f, mpz_t p, mpz_t N, mpz_t go, double *B1done, double B1,
   
   /* Set default B2. See ecm.c for comments */
   if (ECM_IS_DEFAULT_B2(B2))
-    mpz_set_d (B2, B2scale * pow (B1 * PM1_COST, DEFAULT_B2_EXPONENT));
-
+    {
+      if (stage2_variant == 0)
+        mpz_set_d (B2, B2scale * pow (B1 * PM1_COST, DEFAULT_B2_EXPONENT));
+      else
+        mpz_set_d (B2, B2scale * pow (B1 * PM1FS2_COST, 
+                   PM1FS2_DEFAULT_B2_EXPONENT));
+    }
+  
   /* set B2min */
   if (mpz_sgn (B2min) < 0)
     mpz_set_d (B2min, B1);
@@ -867,11 +908,17 @@ pm1 (mpz_t f, mpz_t p, mpz_t N, mpz_t go, double *B1done, double B1,
 
   if (stage2_variant != 0)
     {
-      long P;
+      long P_ntt, P_nontt;
       const unsigned long lmax = 1UL<<28; /* An upper bound */
       unsigned long lmax_NTT, lmax_noNTT;
+      faststage2_param_t params_ntt, params_nontt, *better_params;
 
       mpz_init (faststage2_params.m_1);
+      faststage2_params.l = 0;
+      mpz_init (params_ntt.m_1);
+      params_ntt.l = 0;
+      mpz_init (params_nontt.m_1);
+      params_nontt.l = 0;
 
       /* Find out what the longest transform length is we can do at all.
 	 If no maxmem is given, the non-NTT can theoretically do any length. */
@@ -880,7 +927,7 @@ pm1 (mpz_t f, mpz_t p, mpz_t N, mpz_t go, double *B1done, double B1,
       if (use_ntt)
 	{
 	  unsigned long t;
-	  /* See what transform length that the NTT can handle (due to limited 
+	  /* See what transform length the NTT can handle (due to limited 
 	     primes and limited memory) */
 	  t = mpzspm_max_len (N);
 	  lmax_NTT = MIN (lmax, t);
@@ -890,9 +937,20 @@ pm1 (mpz_t f, mpz_t p, mpz_t N, mpz_t go, double *B1done, double B1,
 	      lmax_NTT = MIN (lmax_NTT, t);
 	    }
 	  outputf (OUTPUT_DEVVERBOSE, "NTT can handle lmax <= %lu\n", lmax_NTT);
+          /* FIXME: if both ntt and no-ntt are tried, but finally ntt is
+             preferred, the last B2 bound computed is that of no-ntt,
+             which is thus wrong */
+          P_ntt = choose_P (B2min, B2, lmax_NTT, k, &params_ntt, 
+                            B2min, B2, 1, ECM_PM1);
+          if (P_ntt != ECM_ERROR)
+            outputf (OUTPUT_DEVVERBOSE, 
+	             "Parameters for NTT: P=%lu, l=%lu\n", 
+	             params_ntt.P, params_ntt.l);
 	}
+      else
+        P_ntt = 0; /* or GCC complains about uninitialized var */
       
-      /* See what transform length that the non-NTT code can handle */
+      /* See what transform length the non-NTT code can handle */
       lmax_noNTT = lmax;
       if (maxmem != 0.)
 	{
@@ -902,18 +960,54 @@ pm1 (mpz_t f, mpz_t p, mpz_t N, mpz_t go, double *B1done, double B1,
 	  outputf (OUTPUT_DEVVERBOSE, "non-NTT can handle lmax <= %lu\n", 
 		   lmax_noNTT);
 	}
+      if (use_ntt != 2)
+        P_nontt = choose_P (B2min, B2, lmax_noNTT, k, &params_nontt, 
+                            B2min, B2, 0, ECM_PM1);
+      else
+        P_nontt = ECM_ERROR;
+      if (P_nontt != ECM_ERROR)
+        outputf (OUTPUT_DEVVERBOSE, 
+                 "Parameters for non-NTT: P=%lu, l=%lu\n", 
+                 params_nontt.P, params_nontt.l);
       
-      P = choose_P (B2min, B2, MAX (lmax_NTT, lmax_noNTT), k, 
-		    &faststage2_params, B2min, B2, use_ntt, ECM_PM1);
-      if (P == ECM_ERROR)
+      if (((!use_ntt || P_ntt == ECM_ERROR) && P_nontt == ECM_ERROR) ||
+          (use_ntt == 2 && P_ntt == ECM_ERROR))
         {
+          outputf (OUTPUT_ERROR, 
+                   "Error: cannot choose suitable P value for your stage 2 "
+                   "parameters.\nTry a shorter B2min,B2 interval.\n");
           mpz_clear (faststage2_params.m_1);
+          mpz_clear (params_ntt.m_1);
+          mpz_clear (params_nontt.m_1);
           return ECM_ERROR;
         }
-      
-      /* See if the selected parameters let us use NTT or not */
-      if (faststage2_params.l > lmax_NTT)
-	use_ntt = 0;
+
+      /* Now decide wether to take NTT or non-NTT.
+         How to choose the better one is not an easy question.
+         It will depend on the speed ratio between NTT/non-NTT code,
+         their difference in memory use and available memory.
+         For now, we choose the one that uses a longer transform length.
+         FIXME: Write something not brain-dead here */
+      if (use_ntt == 0 || P_ntt == ECM_ERROR ||
+          (use_ntt == 1 && params_nontt.l > params_ntt.l))
+        {
+          better_params = &params_nontt;
+          use_ntt = 0;
+        }
+      else
+        {
+          better_params = &params_ntt;
+          use_ntt = 1;
+        }
+
+      faststage2_params.P = better_params->P;
+      faststage2_params.s_1 = better_params->s_1;
+      faststage2_params.s_2 = better_params->s_2;
+      faststage2_params.l = better_params->l;
+      mpz_set (faststage2_params.m_1, better_params->m_1);
+
+      mpz_clear (params_ntt.m_1);
+      mpz_clear (params_nontt.m_1);
       
       if (maxmem != 0.)
 	  outputf (OUTPUT_VERBOSE, "Using lmax = %lu with%s NTT which takes "
@@ -977,7 +1071,7 @@ pm1 (mpz_t f, mpz_t p, mpz_t N, mpz_t go, double *B1done, double B1,
   
   /* Print B1, B2, polynomial and x0 */
   print_B1_B2_poly (OUTPUT_NORMAL, ECM_PM1, B1, *B1done, B2min_parm, B2min, 
-		    B2, (stage2_variant == 0) ? root_params.S : 1, p, 0);
+		    B2, (stage2_variant == 0) ? root_params.S : 1, p, 0, NULL);
 
   /* If we do a stage 2, print its parameters */
   if (mpz_cmp (B2, B2min) >= 0)
@@ -992,6 +1086,22 @@ pm1 (mpz_t f, mpz_t p, mpz_t N, mpz_t go, double *B1done, double B1,
                  dF, k, root_params.d1, root_params.d2, root_params.i0);
     }
 
+  if (test_verbose (OUTPUT_VERBOSE))
+    {
+      if (mpz_sgn (B2min_parm) >= 0)
+        {
+          outputf (OUTPUT_VERBOSE, 
+            "Can't compute success probabilities for B1 <> B2min\n");
+        }
+      else
+        {
+          rhoinit (256, 10);
+          print_prob (B1, B2, dF, k, 
+                      (stage2_variant == 0) ? root_params.S : 1, go);
+        }
+    }
+
+
   mpres_init (x, modulus);
   mpres_set_z (x, p, modulus);
 
@@ -1029,6 +1139,12 @@ pm1 (mpz_t f, mpz_t p, mpz_t N, mpz_t go, double *B1done, double B1,
                         use_ntt, TreeFilename, stop_asap);
     }
 
+  if (test_verbose (OUTPUT_VERBOSE))
+    {
+      if (mpz_sgn (B2min_parm) < 0)
+        rhoinit (1, 0); /* Free memory of rhotable */
+    }
+
 clear_and_exit:
   mpres_get_z (p, x, modulus);
   mpres_clear (x, modulus);
