@@ -1,6 +1,7 @@
 /* The 'P+1' algorithm.
 
-  Copyright 2002, 2003, 2004, 2005 Paul Zimmermann and Alexander Kruppa.
+  Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008
+  Paul Zimmermann and Alexander Kruppa.
 
   This file is part of the ECM Library.
 
@@ -214,7 +215,7 @@ pp1_stage1 (mpz_t f, mpres_t P0, mpmod_t n, double B1, double *B1done,
   /* then all primes > sqrt(B1) and taken with exponent 1 */
   for (; p <= B1; p = getprime ())
     {
-      pp1_mul_prac (P0, (unsigned long) p, n, P, Q, R, S, T);
+      pp1_mul_prac (P0, (ecm_uint) p, n, P, Q, R, S, T);
   
       if (stop_asap != NULL && (*stop_asap) ())
         {
@@ -241,9 +242,6 @@ pp1_stage1 (mpz_t f, mpres_t P0, mpmod_t n, double B1, double *B1done,
     *B1done = p;
   
   mpres_sub_ui (P, P0, 2, n);
-#ifndef FULL_REDUCTION
-  mpres_normalize (P); /* needed for gcd */
-#endif
   mpres_gcd (f, P, n);
   youpi = mpz_cmp_ui (f, 1);
 
@@ -330,7 +328,7 @@ pp1_mul2 (mpres_t a, mpres_t b, mpres_t P, mpz_t e, mpmod_t n)
       /* square: (ax+b)^2 = (a^2P+2ab) x + (b^2-a^2) */
       mpres_mul (t, a, a, n); /* a^2 */
       mpres_mul (a, a, b, n);
-      mpres_mul_ui (a, a, 2, n); /* 2ab */
+      mpres_add (a, a, a, n); /* 2ab */
       mpres_mul (b, b, b, n); /* b^2 */
       mpres_sub (b, b, t, n); /* b^2-a^2 */
       mpres_mul (t, t, P, n); /* a^2P */
@@ -538,8 +536,8 @@ pp1_rootsF (listz_t F, root_params_t *root_params, unsigned long dF,
                   */
 		  mpres_mul (u, state.fd[params->next * (params->S + 1)].x, x[0],
 			     modulus);
-		  mpres_mul_ui (v, state.fd[params->next * (params->S + 1)].y,
-				2, modulus);
+		  mpres_add (v, state.fd[params->next * (params->S + 1)].y,
+                             state.fd[params->next * (params->S + 1)].y, modulus);
 		  mpres_add (u, u, v, modulus);
 		  mpres_get_z (F[i++], u, modulus);
 		}
@@ -746,8 +744,8 @@ pp1_rootsG (listz_t G, unsigned long dF, pp1_roots_state_t *state,
             {
               mpres_mul (u, state->fd[params->next * (params->S + 1)].x, x[0],
                          modulus);
-              mpres_mul_ui (v, state->fd[params->next * (params->S + 1)].y,
-                            2, modulus);
+              mpres_add (v, state->fd[params->next * (params->S + 1)].y,
+                         state->fd[params->next * (params->S + 1)].y, modulus);
               mpres_add (u, u, v, modulus);
               mpres_get_z (G[i++], u, modulus);
             }
@@ -823,7 +821,13 @@ pp1 (mpz_t f, mpz_t p, mpz_t n, mpz_t go, double *B1done, double B1,
 
   /* Set default B2. See ecm.c for comments */
   if (ECM_IS_DEFAULT_B2(B2))
-    mpz_set_d (B2, B2scale * pow (B1 * PP1_COST, DEFAULT_B2_EXPONENT));
+    {
+      if (stage2_variant == 0)
+        mpz_set_d (B2, B2scale * pow (B1 * PP1_COST, DEFAULT_B2_EXPONENT));
+      else
+        mpz_set_d (B2, B2scale * pow (B1 * PP1FS2_COST, 
+                   PM1FS2_DEFAULT_B2_EXPONENT));
+    }
 
   /* set B2min */
   if (mpz_sgn (B2min) < 0)
@@ -841,6 +845,7 @@ pp1 (mpz_t f, mpz_t p, mpz_t n, mpz_t go, double *B1done, double B1,
       unsigned long lmax_NTT, lmax_noNTT;
       
       mpz_init (faststage2_params.m_1);
+      faststage2_params.l = 0;
       
       /* Find out what the longest transform length is we can do at all.
 	 If no maxmem is given, the non-NTT can theoretically do any length. */
@@ -885,6 +890,9 @@ pp1 (mpz_t f, mpz_t p, mpz_t n, mpz_t go, double *B1done, double B1,
 		    &faststage2_params, B2min, B2, use_ntt, ECM_PP1);
       if (P == ECM_ERROR)
 	{
+          outputf (OUTPUT_ERROR, 
+                   "Error: cannot choose suitable P value for your stage 2 "
+                   "parameters.\nTry a shorter B2min,B2 interval.\n");
 	  mpz_clear (faststage2_params.m_1);
 	  return ECM_ERROR;
 	}
@@ -943,7 +951,7 @@ pp1 (mpz_t f, mpz_t p, mpz_t n, mpz_t go, double *B1done, double B1,
 
   /* Print B1, B2, polynomial and x0 */
   print_B1_B2_poly (OUTPUT_NORMAL, ECM_PP1, B1, *B1done, B2min_parm, B2min, 
-		    B2, (stage2_variant == 0) ? root_params.S : 1, p, 0);
+		    B2, (stage2_variant == 0) ? root_params.S : 1, p, 0, NULL);
 
   /* If we do a stage 2, print its parameters */
   if (mpz_cmp (B2, B2min) >= 0)
@@ -962,12 +970,12 @@ pp1 (mpz_t f, mpz_t p, mpz_t n, mpz_t go, double *B1done, double B1,
   mpres_init (a, modulus);
   mpres_set_z (a, p, modulus);
 
-  /* since pp1_mul_prac takes an unsigned long, we have to check
-     that B1 <= MAX_ULONG */
-  if (B1 > (double) ULONG_MAX)
+  /* since pp1_mul_prac takes an ecm_uint, we have to check
+     that B1 <= ECM_UINT_MAX */
+  if (B1 > (double) ECM_UINT_MAX)
     {
       outputf (OUTPUT_ERROR, "Error, maximal step1 bound for P+1 is %lu\n", 
-               ULONG_MAX);
+               ECM_UINT_MAX);
       youpi = ECM_ERROR;
       goto clear_and_exit;
     }
